%%
% cswibra: branch switching via Cubic BE (+storage of kernel in p.mat.ker for
% taugen) 
%
% compute coefficients for cubic bif eqns (cbe), 
% provide guesses [b,alv] and solve qbe with fsolve
% From each soln [b,alv] generate tangent and store in p.mat.ctau
% Then use seltau to select a bif. direction and call cont 
% 
% auxiliary arguments in aux, i.e., 
% mu2:     assume evals with |mu|<mu2 as zero, default p.nc.mu2
% m:       give number of zero evals 
% ral:     if 1, then use random guesses for alv 
% alc:     guesses for al to generate guess alv (if ral~=1) 
% al0v:    guesses for al0 
% ftol:    tolerance for solving cbe
% 
function q=cswibra3(dir,fname,ndir,varargin)
p=loadp(dir,fname);  p=setfn(p,ndir); fss=p.plot.fs; p.plot.fs=12; 
if p.sol.ptype==3; fprintf('\n Hopf-point, use hoswibra\n'); q=[]; return; end
u=p.u; lam=getlam(p); tau=p.tau; xi=p.sol.xi; n=p.nu; 
aux=[]; if nargin>3; aux=varargin{1}; end                  % read aux arguments 
try del=aux.del; catch; del=1e-6; end; deli=1/del; % don't choose del too small!
try pstyle=aux.pstyle; catch; pstyle=p.plot.pstyle; end; try verb=aux.verb; catch verb=p.sw.verb; end 
try p.sol.ds=aux.ds; catch;  p.sol.ds=p.nc.dsmax/10; end; 
try mu2=aux.mu2; catch; mu2=p.nc.mu2; end; try p.plot.fs=aux.fs; catch; p.plot.fs=12; end; 
fprintf(['lam=' num2str(lam), '; 4 smallest eigenvalues: ' num2str(p.sol.muv(1:4)) '\n']); 
if isfield(aux,'m'); m=aux.m;   % user provided multiplicity
else; m=0; for i=1:length(p.sol.muv); if abs(p.sol.muv(i))<mu2; m=m+1; end; end % compute multiplicity 
if m==0; fprintf('no zero evals\n'); q=0; return; end
end
fprintf(['using m=' num2str(m),'\n']); %m=2; 
r=resi(p,u); [Gu,Glam]=getder(p,u,r);  vs=size(Gu,1); % residual and jacobian
opts.v0=ones(vs,1)/vs; opts.disp=0; M=p.mat.M; %M=speye(n); 
[phiv,mu]=eigs(Gu,M,m,0,opts); phiv=real(phiv); phiv=hugram(phiv);  % getting EVecs
[psiv,mu]=eigs(Gu',M,m,0,opts); psiv=real(psiv); psiv=hugram2(psiv,phiv); % adjoints 
if verb>0
  set(0,'Units','pixels'); scnsize = get(0,'ScreenSize'); 
  scnw=scnsize(3); scnh=scnsize(4); sz=scnh/3-scnh*5/100; 
  for i=1:m
      switch mod(i,3); case 1; pheight=2*scnh/3; case 2; pheight=sz+115; case 0; pheight=0; end;
      pw=ceil(i/3); pw=pw+1; set(figure(10+i),'Position',[scnw-pw*sz pheight sz sz]);
      plotsolu(p,phiv(:,i),10+i,1,pstyle); title(['phi-' mat2str(i,2)]); 
  end 
end 
drawnow; mt=zeros(p.nu+1,m); % store kernel vectors for taugen in p.mat.tau 
for i=1:m; tau1=[phiv(:,i);0]; tau1=tau1/xinorm(tau1,xi,p.nc.nq,p.sol.xiq); mt(:,i)=tau1; end; p.mat.ker=mt; 
qjk=qjkv(p,Gu,u,phiv,del);  % qij:=Guu[phi_i,phi_j]
n1=-0.5*(Gu\reshape(qjk(1,1,:),n,1)); n1=proNperp(n1,psiv,phiv); % nij=Gu\qij and n0=Gu\Glam 
n2=-0.5*(Gu\reshape(qjk(2,2,:),n,1)); n2=proNperp(n2,psiv,phiv);
n3=-0.5*(Gu\reshape(qjk(3,3,:),n,1)); n3=proNperp(n3,psiv,phiv);
n4=-(Gu\reshape(qjk(1,2,:),n,1)); n4=proNperp(n4,psiv,phiv); 
n5=-(Gu\reshape(qjk(1,3,:),n,1)); n5=proNperp(n5,psiv,phiv); 
n6=-(Gu\reshape(qjk(2,3,:),n,1)); n6=proNperp(n6,psiv,phiv); 
n7=-(Gu\Glam); n7=proNperp(n7,psiv,phiv); 
nv=[n1 n2 n3 n4 n5 n6 n7]; % max(max(abs(nv(:))))
fprintf('eta-norms: %g, %g, %g, %g\n',max(abs(n1(:))),max(abs(n2(:))),max(abs(n3(:))),max(abs(n4(:)))); 
cjkl=cjklv(p,u,phiv,del);     % cjkl=Guuu[phi_j,phi_j,phi_k] (m x m x m matrix of n vectors) 
bjk=bjkv3(p,Gu,u,phiv,del,nv); % bjk =Guu[phi_j,n_k] (n x (10+3)) matrix 
dijkl=zeros(m,m,m,m);         % for cubic coeff.
gij=zeros(m,m); % for G_uu[u',w], beta-part of w 
fij=zeros(m,m); % for G_{u,lam}[u',lam'']
for i=1:m
    for j=1:m
        gij(i,j)=psiv(:,i)'*bjk(:,10+j);  % coeffis of beta*al_j from G_uu[u',w]  
        up=u; up(1:p.nu)=up(1:p.nu)+del*phiv(:,j); 
        rp=resi(p,up); Glamup=getGlam(p,up,rp); Glamd=deli*(Glamup-Glam); 
        fij(i,j)=psiv(:,i)'*Glamd; % coeffis of beta*al_j from G_u,lam[u',lam'']
        for k=1:m
            for l=1:m
                dijkl(i,j,k,l)=psiv(:,i)'*reshape(cjkl(j,k,l,:),n,1); % cubics 
            end
        end
    end
    % add corrections from G_uu[u',w] to cubics 
    fak=6; 
    dijkl(i,1,1,1)=dijkl(i,1,1,1)+fak*psiv(:,i)'*bjk(:,1); % al1^3
    dijkl(i,1,2,2)=dijkl(i,1,2,2)+fak*psiv(:,i)'*bjk(:,2); % al1*al2^2
    dijkl(i,1,3,3)=dijkl(i,1,3,3)+fak*psiv(:,i)'*bjk(:,3); % al1*al3^2
    dijkl(i,1,1,2)=dijkl(i,1,1,2)+fak*psiv(:,i)'*bjk(:,4); % al1^2*al2
    dijkl(i,1,1,3)=dijkl(i,1,1,3)+fak*psiv(:,i)'*bjk(:,5); % al1^2*al3
    dijkl(i,1,2,3)=dijkl(i,1,2,3)+fak*psiv(:,i)'*bjk(:,6); % al1*al2*al3
    dijkl(i,2,2,2)=dijkl(i,2,2,2)+fak*psiv(:,i)'*bjk(:,7); % al2^3
    dijkl(i,2,3,3)=dijkl(i,2,3,3)+fak*psiv(:,i)'*bjk(:,8); % al2*al3^2
    dijkl(i,2,2,3)=dijkl(i,2,2,3)+fak*psiv(:,i)'*bjk(:,9); % al2^2*al3
    dijkl(i,3,3,3)=dijkl(i,3,3,3)+fak*psiv(:,i)'*bjk(:,10);% al3^3 
end
%gij, fij, max(abs(dijkl(:)))
try bet0=aux.bet0; catch; bet0=0.01; end; % read more aux arguments 
try ral=aux.ral; catch; ral=0; end;       % random alpha-trials? 
try alc=aux.alc; catch; alc=[0 1 -1]; end; try ftol=aux.ftol; catch; ftol=1e-18; end 
try jactol=aux.jactol; catch; jactol=1e-10; end; try almin=aux.almin; catch; almin=1e-6; end  
alv=[]; nsol=0; opt=optimoptions('fsolve','Display','off','TolFun',ftol); % ftol, pause
%opt=optimoptions(opt,'Algorithm','trust-region'); 
if ral || m>3; alsv=(rand(10,m)-0.5); alsv=[alsv; eye(m)]; % alpha start-vectors for fsolve 
else; alsv=[]; % 0 and 1 (up to m=3) 
  for i=1:length(alc); 
    if m==1; alsv=[alsv; alc(i)]; 
    else
      for j=1:length(alc)
        if m==2; new_al=[alc(i) alc(j)]; 
          if any(new_al); alsv=[alsv; new_al]; end 
          else; for k=1:length(alc); new_al=[alc(i) alc(j) alc(k)]; 
                  if any(new_al); alsv=[alsv; new_al]; end
              end
        end
      end; 
   end; 
  end;
end 
%alsv, pause
p.mat.ctau=zeros(p.nu+1,20);  % storage for tangents 
alliso=1; 
fu=@(al) cbe(al,dijkl,gij,fij,bet0,m);   
for i=1:size(alsv,1)
   alstart=alsv(i,:); 
   %r=cbe(alstart,dijkl,gij,fij,bet0,m)', 
   [al,fval,flag,out,jac]=fsolve(fu,alstart,opt); djac=det(jac); iso=1; % SOLVE CBE  
   if abs(djac)<jactol; %fprintf('non-isolated\n'); 
       iso=0; alliso=0; end
   if iso && (norm(fval)^2<10*ftol) && (norm(al)>almin); %fprintf('sol found,  '); al, 
     new=1; % check if new   
     for ii=1:nsol; 
       old1=abs(al*alv(ii,:)') > 0.99*norm(al)*norm(alv(ii,:)); 
       if old1; new=0; break; end 
     end
     if new
       tau=[0*phiv(:,1);0]; for it=1:m; tau(1:p.nu)=tau(1:p.nu)+al(it)*phiv(:,it); end
       tau=tau/xinorm(tau,xi,p.nc.nq,p.sol.xiq); 
       plotsolu(p,[tau;p.u(p.nu+1:end)],6,p.plot.pcmp,pstyle); 
       nsol=nsol+1; fprintf('%i ',nsol); alv=[alv;al]; %size(tau), nsol
       p.mat.qtau(:,nsol)=tau; % store tangent
     end 
   end
end
fprintf('\n'); 
if ~alliso; fprintf('found non-isolated zeros, try gentau for tangent guesses.\n'); end 
fprintf('found %i candidates, alpha-values: \n',nsol); 
for i=1:min(nsol,10); fprintf('%2i  ',i); disp(alv(i,1:m)); end 
if verb>0
  for i=1:min(nsol,9) 
    switch mod(i,3); case 1; pheight=2*scnh/3; case 2; pheight=sz+115; case 0; pheight=0; end;     
    pw=ceil(i/3); set(figure(20+i),'Position',[scnw-pw*sz pheight sz sz]);
    up=p.mat.qtau(:,i); m1=min(up(1:p.np)); m2=max(up(1:p.np)); 
    d=0.01/(1+m2-m1); 
    plotsolu(p,up,20+i,1,pstyle); title(['lam´=' mat2str(p.mat.qtau(end,i),2)]); xlabel(''); ylabel(''); 
   [po,~,~]=getpte(p); if size(po,1)==2 && pstyle==2; caxis([m1-d,m2+d]); end 
  end 
end
p.tau=tau; p.sol.alv=alv; bs=p.branch(:,size(p.branch,2)); p=resetc(p);
p.sol.ptype=-2; bs(1)=0; bs(2)=p.sol.ptype; % put last of p on new branch
p.branch=bs; p.sol.deta=0;  p.plot.fs=fss;  q=p; 